#######################################################
#
#	Dump of installed site application's information
#
#######################################################

package SiteApp;

use strict;
use PleskVersion;

my $dbh;

# Compile-time fields checking commented for work with Perl 5.0
#use fields qw(id
#			  error
#			  SiteApp
#			  Parameters
#			  Resources
#			  Package
#			  htdocs
#			  cgi_bin);

##################################################
# Parameters:
# $site_application_id, $dbh
##################################################
sub new( $ $ )
{
	my $class = shift;

	my $self = {};
	bless($self, $class);

	$self->{id} = shift;
	$dbh = shift;

	$self->{SiteApp}    = {};
	$self->{Parameters} = {};
	$self->{Resources}  = {};
	$self->{Package}    = {};
	$self->{htdocs}  = [];
	$self->{cgi_bin} = [];

	my $sql = "SELECT * FROM SiteApps WHERE id='" . $self->{id} . "'";
	if (!$dbh->execute_rownum($sql)) {
		$self->_sqlError("Can not find Site Application (ID=" . $self->{id} . ")");
		return $self;
	}

	$self->{SiteApp} = $dbh->fetchhash();
	$dbh->finish();

	$self->_getParams();
	$self->_getLinkedResources();
	$self->_getPackageSpec();
	$self->_getHtdocsFiles();
	$self->_getCgibinFiles();

	return $self;
}

sub getName
{
	my SiteApp $self = shift;
	return $self->{Package}{'name'};
}

sub getVersion
{
	my SiteApp $self = shift;
	return $self->{Package}{'version'};
}

sub getRelease
{
	my SiteApp $self = shift;
	return $self->{Package}{'release'};
}

sub getDescription
{
	my SiteApp $self = shift;
	return $self->{Package}{'description'};
}

sub isCommercial
{
	my SiteApp $self = shift;
	return ($self->{Package}{'access_level'} & 1) != 0;
}

sub isIntegrated
{
	my SiteApp $self = shift;
	return 'true' eq $self->{Package}{'integrated'};
}

# Returns %hash
sub getParams
{
	my SiteApp $self = shift;
	return $self->{Parameters};
}

# Returns @list
sub getDatabases
{
	my SiteApp $self = shift;
	if (exists($self->{Resources}{'database'})) {
		return @{$self->{Resources}{'database'}};
	}
	return;
}

# Returns @list
sub getCustomButtons
{
	my SiteApp $self = shift;
	if (exists($self->{Resources}{'custom_button'})) {
		return @{$self->{Resources}{'custom_button'}};
	}
	return;
}

sub getInstallPrefix
{
	my SiteApp $self = shift;
	return $self->{SiteApp}{'install_prefix'};
}

# Returns true if site app is installed on domain
sub isDomainSiteapp
{
	my SiteApp $self = shift;
	return 'domain' eq $self->{SiteApp}{'dom_type'};
}

# Returns true if site app is installed on the given subdomain
sub isSubdomainSiteapp( $ )
{
	my SiteApp $self = shift;
	my ($subdom_id) = @_;
	return ('subdomain' eq $self->{SiteApp}{'dom_type'}) && ($subdom_id == $self->{SiteApp}->{'dom_id'});
}

# Returns true if site application is installed into ssl directory
sub isSsl
{
	my SiteApp $self = shift;
	return 'httpsdocs' eq $self->{SiteApp}{'htdocs_directory'};
}

# Returns list of files from htdocs directory, belonging
# to site application. Uses paths relative to htdocs.
# Returns @list
sub getHtdocsFiles
{
	my SiteApp $self = shift;
	return map {$self->{SiteApp}{'install_prefix'} . "/$_"} @{$self->{htdocs}};
}

# Returns list of files from cgi-bin directory, belonging
# to site application. Uses paths relative to cgi-bin.
# Returns @list
sub getCgibinFiles
{
	my SiteApp $self = shift;
	return map {$self->{SiteApp}{'install_prefix'} . "/$_"} @{$self->{cgi_bin}};
}

# Returns error message, indicating erroneous state of the object
sub error
{
	my SiteApp $self = shift;
	return $self->{error};
}

sub _getHtdocsFiles
{
	my SiteApp $self = shift;
	# Prior to 7.5.4 there were no SiteAppFiles table
	if (!PleskVersion::atLeast(7, 5, 4)) {
		$self->{htdocs} = ['.'];
		return $self->{htdocs};
	}

	# TODO: The following problem should be addressed:
	# 
	# If site application creates directories for data storage in run-time
	# they are not listed in SiteAppFiles table, and therefore not backuped together
	# with site application. But they also will not be backuped together with hosting
	# content, as site application owns their directory. Hosting content backup
	# does not include files/directories, owned by site application.
	# As a workaround we'll register all files in site app directory as files of the
	# given siteapp (for non / directories)
	# 
	# Proper solution is one of:
	# 1. backup site app content and list of site app files separately
	# 2. include unregistered files from siteapp directory into hosting content backup
	#if ($self->{SiteApp}{'install_prefix'} ne '.') {
		$self->{htdocs} = ['.'];
		return $self->{htdocs};
	#}

	my $sql = "SELECT file FROM SiteAppFiles WHERE instance_id='" . $self->{id} . "' AND prefix='htdocs'";
	if ($dbh->execute_rownum($sql)) {
		while (my $ptrRow = $dbh->fetchrow()) {
			push @{$self->{htdocs}}, $ptrRow->[0];
		}
	}
	$dbh->finish();

	return $self->{htdocs};
}

sub _getCgibinFiles
{
	my SiteApp $self = shift;
	# Prior to 7.5.4 there were no SiteAppFiles table
	if (!PleskVersion::atLeast(7, 5, 4)) {
		$self->{cgi_bin} = ['.'];
		return $self->{cgi_bin};
	}

	my $sql = "SELECT file FROM SiteAppFiles WHERE instance_id='" . $self->{id} . "' AND prefix='cgi-bin'";
	if ($dbh->execute_rownum($sql)) {
		while (my $ptrRow = $dbh->fetchrow()) {
			push @{$self->{cgi_bin}}, $ptrRow->[0];
		}
	}
	$dbh->finish();

	return $self->{cgi_bin};
}

sub _sqlError( $ )
{
	my SiteApp $self = shift;
	my ($message) = @_;
	$dbh->finish();
	$self->{error} = $message;
}

sub _getParams
{
	my SiteApp $self = shift;
	my $sql = "SELECT parameter, value FROM Parameters WHERE id='" . ${$self->{SiteApp}}{'params_id'} . "'";
	if ($dbh->execute_rownum($sql)) {
		while (my $ptrRow = $dbh->fetchrow()) {
			$ptrRow->[1] ||= ''; #NULL => ''
			$self->{Parameters}{$ptrRow->[0]} = $ptrRow->[1];
		}
		$dbh->finish();
	}
}

sub _getLinkedResources
{
	my SiteApp $self = shift;
	my $sql = "SELECT * FROM SiteAppResources WHERE app_id='" . $self->{id} . "'";
	if ($dbh->execute_rownum($sql)) {
		while (my $ptrRow = $dbh->fetchhash()) {
			push @{$self->{Resources}{$ptrRow->{'type'}}}, {'id' => $ptrRow->{'res_id'}, 'param' => defined $ptrRow->{'res_param_1'} ? $ptrRow->{'res_param_1'} : undef};
		}
		$dbh->finish();
	}
}

sub _getPackageSpec
{
	my SiteApp $self = shift;
	my $sql = "SELECT * FROM SiteAppPackages WHERE id='" . $self->{SiteApp}{'app_id'} . "'";
	if (!$dbh->execute_rownum($sql)) {
		return $self->_sqlError("Can not find package for Site Application (ID=" . $self->{id} . ")");
	}
	$self->{Package} = $dbh->fetchhash();
	$dbh->finish();
}

1;
