#!/usr/bin/perl

# use for dump of user's mysql databases
use vars qw($storage);

use vars qw($dumpDir $ptrArgs $workDir);
use strict;

use XmlNode;

my $DEFAULT_RESELLER='root';

my %mapLimits = ('max_addon'=>'MAXADDON',
				 'max_parked'=>'MAXPARK',
				 'max_ftpusers'=>'MAXFTP',
				 'max_maillists'=>'MAXLST',
				 'max_box'=>'MAXPOP',
				 'max_db'=>'MAXSQL',
				 'max_subdom'=>'MAXSUB',
				);


#
# parse command line
#
my %arg_opts = ('--help|-h'=>'',
				'--config|-c'=>'s',
				'--get-status|-s'=>'',
				'--dump-accounts|-dc'=>'s',
				'--dump-domains|-dd'=>'s',
				'--dump-all|-da'=>'',
				'--get-content-list|-lc'=>'',
				'--no-content|-nc'=>'',
				'--no-compress|-nz'=>'',
				'--output|-o'=>'s',
				'--status-file|-sf'=>'s',
			   );

my $noContent = 0;

#
# global variables
#
my ($ptrRowResellers,%resellers,@default_reseller_account,
	$ptrQuota,$ptrResellerInfo, %servers_ips);
my $ptrUserdomainsHash;
my %trueUserDomains;
my %anonftps;
#
# end global variables
#


#@@INCLUDE FILE="agent.include.pl"@@
if (-f 'agent.include.pl') {
  require 'agent.include.pl';
} else {
  require '../agent.include.pl';
}
#@@/INCLUDE@@

$ptrArgs = &getArguments (\@ARGV,\%arg_opts);

my ($outPath,$dumpFile,$statusFile);

$workDir = AgentConfig::cwd();

$dumpFile = $workDir.'/dump.xml';
$statusFile = $workDir.'/dumping-status.xml';
$dumpDir = $workDir.'/pma';

my $objDumpStatus = &makeDumpStatus($ptrArgs->{'status-file'}||$statusFile);

my (%cPanelConfig);
my $rootName = 'cPanel9dump';
my $dtdName = 'cPanel9.dtd';

unless ($ptrArgs->{'config'} = &loadMainConfig($ptrArgs->{'config'},\%cPanelConfig)) {
  if (exists $ptrArgs->{'get-status'}) {
    printAgentStatus();
	exit 0;
  } else {

	if (exists ($ptrArgs->{'dump-all'}) ||
		exists ($ptrArgs->{'dump-accounts'}) ||
		exists ($ptrArgs->{'dump-domains'})) {

	  my $root=XmlNode->new($rootName);

	  if (ref($root)=~/HASH/ && ref($root->{'PRINT'})=~/CODE/) {
		my $outFh = &openOutput($ptrArgs->{'output'});
		&printXml($root,$outFh,$dtdName);
		&closeOutput();
	  }
	}

	die "Error: cPanel config file 'cpanel.conf' is not found\n";
  }
}
my $installDir = $cPanelConfig{'installDir'}||'/var/cpanel';

my $wrapUserMysql;

#
# get MIME Base64 encoder
#
my $wrapBase64 = makeMIMEBase64();



if (exists $ptrArgs->{'get-status'}) {
  printAgentStatus();
  exit 0;
} elsif (exists ($ptrArgs->{'dump-all'}) ||
		 exists ($ptrArgs->{'dump-accounts'}) ||
		 exists ($ptrArgs->{'dump-domains'})) {

  my ($root,@accounts,@domains,$ptrAccounts,$ptrDomains, $value);

  initStorage();

  getDumpDir($dumpDir);

  if (exists $ptrArgs->{'no-compress'}) {
	setCompress(0);
  }

  &printToLog("Work dir: $workDir");
  &printToLog("Dump file: $dumpFile");
  &printToLog("Status file: ".$objDumpStatus->{'FILE'}->());


  if ($value = $ptrArgs->{'dump-accounts'}) {
	if ($value eq "-") {
	  $value = <STDIN>;
	  chomp $value;
	}
	@accounts = split(/\s*,\s*/,$value);
	$ptrAccounts = \@accounts;
  }

  if ($value = $ptrArgs->{'dump-domains'}) {
	if ($value eq "-") {
	  $value = <STDIN>;
	  chomp $value;
	}
	@domains = split(/\s*,\s*/,$value);
	$ptrDomains = \@domains;
  }
  #
  # generate a xml dump
  #

  $root = &getcPanel9Dump($dumpDir, $ptrAccounts, $ptrDomains, $rootName);

  #
  # print dump to output
  #
  $storage->finish($root);
} elsif (exists $ptrArgs->{'get-content-list'}) {
  makeContentList();
} else {
  &printHelp();
}

exit 0;

#######################################################
#
#  Get all client accounts, registered in the system
#
#######################################################
sub getAllAccounts {
  my @all_accounts;

  opendir(USRS, "$installDir/users");
  my @accts = readdir(USRS);
  closedir(USRS);
  chomp(@accts);

  # remove duplicate and possible wrong entries
  foreach my $account (@accts) {
	next if ($account =~ /\./ || grep(/^$account$/, @all_accounts));
	push @all_accounts, $account;
  }

  return @all_accounts;
}

##########################################
#
#   $ptrAccounts - accounts, that have been specified as a --dump-accounts
#   $ptrDomains  - domains, that have been specified as a --dump-domains
#
##########################################
sub getcPanel9Dump {

  my ($dumpDir, $ptrAccounts, $ptrDomains, $rootName) = @_;

  my $root = XmlNode->new($rootName, 'attributes' => {'agent-name' => 'cPanel9'});

  my($reseller, $fileResellers, $ptrRows, $owner, $item, $account,
	 $ptrHash, $key, $value);

  my @all_accounts = getAllAccounts();

  # No accounts - nothing to dump
  return $root if (-1 == $#all_accounts);

  if ( -f "$installDir/resellers") {
	# Find all the resellers in the system
	$fileResellers = &makeFileParser("$installDir/resellers",
									 'KEYSEPARATOR' => ':',
									 'VALUESEPARATOR' => ',',
									 'KEYVALUESEPARATOR' => ' ');

	# All the resellers in the system (not including the default reseller)
	$ptrRowResellers = $fileResellers->{'PARSE'}->();
  }

  # Associative array of domain owners => {0 if only this domain should
  # be dumped and 1 if all domains should be dumped, domain}
  my %accounts;


  # If domains were specified through command line search their owning accounts
  if (ref($ptrDomains) =~ /ARRAY/) {

	foreach my $domain (@{$ptrDomains}) {
	  if ($owner = &getDomainOwner($domain)) {
		if (exists($accounts{$owner})) {
		  push @{$accounts{$owner}->[1]}, $domain;
		} else {
		  $accounts{$owner} = [0, [$domain]];
		}
	  }
	}
  }

  # If accounts were specified through command line - mark them for dumping
  if (ref($ptrAccounts) =~ /ARRAY/) {
	foreach my $account (@{$ptrAccounts}) {
	  $accounts{$account} = [1, undef];
	}
  }


  # Check that all found resellers are exists
  foreach my $reseller (keys %{$ptrRowResellers}) {
	unless(-T "$installDir/users/$reseller") {
	  delete $ptrRowResellers->{$reseller};
	  next;
	}
  }

  # If no accounts/domains were specified through command line $checkAccounts is 0
  my $checkAccounts = scalar(%accounts);


  # For each reseller in the system
  foreach $account (@all_accounts) {

	my $fileUser = makeFileParser("$installDir/users/$account");

	next unless ($fileUser);

	$ptrRows = $fileUser->{'PARSE'}->('KEYSEPARATOR' => '=');

	# If file is empty or wrong format
	unless (ref($ptrRows) =~ /HASH/) {
	  delete $ptrRowResellers->{$account};
	  next;
	}

	$owner = $ptrRows->{'OWNER'};

	# If account belongs to first-level reseller
	if ($owner && ($owner ne $account) &&
		($owner ne $DEFAULT_RESELLER) &&
		(exists $ptrRowResellers->{$owner})) {
	  push @{$resellers{$owner}}, $account; # customer
	  # If account was specified through the command line
	  if ($checkAccounts &&
		  exists ($accounts{$account}) &&
		  !exists ($accounts{$owner})
		 ) {					## account
		$accounts{$owner} = [0, undef];	# account drag reseller
	  }
	} elsif ($owner eq $account || $owner eq $DEFAULT_RESELLER) {
	  # If reseller belongs to default reseller

	  unless (exists $resellers{$account}) {
		$resellers{$account} = [];
	  }
	} else {
	  if (!$checkAccounts || exists($accounts{$account})) {
		push @default_reseller_account, $account;
	  }
	}
  }


  #
  # prepare --dump-accouns
  #
  if ($checkAccounts) {

	my (@accRes);
	foreach $reseller (keys %resellers) {
	  unless (exists($accounts{$reseller})) {
		delete $resellers{$reseller};
		next;
	  }

	  unless ($accounts{$reseller}->[0]) { ## not full
		@accRes = ();
		foreach $account (@{$resellers{$reseller}}) {
		  if (exists($accounts{$account})) {
			push @accRes,$account;
		  }
		}
		$resellers{$reseller}=[@accRes];
	  }
	}
  }

  my $fileUser = makeFileParser("/etc/quota.conf");
  # Get disk quotas
  $ptrQuota = $fileUser->{'PARSE'}->('KEYSEPARATOR' => '=',
									 'VALUESEPARATOR' => '');

  unless (ref($ptrQuota) =~ /HASH/) {
	$ptrQuota = {};
  }

  #
  # prepare reseller's info
  #
  $ptrResellerInfo={};

  if ( -f "$installDir/reseller-limits-bynumber") {
	$fileUser = makeFileParser("$installDir/reseller-limits-bynumber");
	$ptrHash = $fileUser->{'PARSE'}->('KEYSEPARATOR' => '=', 'VALUESEPARATOR' => '');

	if (ref($ptrHash) =~ /HASH/) {
	  while (($reseller, $value) = each(%{$ptrHash})) {
		$ptrResellerInfo->{$reseller} = {'limits-bynumber' => $value};
	  }
	}
  }


  if ( -f "$installDir/reseller-resource-limits") {
	$fileUser = makeFileParser("$installDir/reseller-resource-limits");
	$ptrHash = $fileUser->{'PARSE'}->('KEYSEPARATOR' => ' ', 'VALUESEPARATOR' => '');

	if (ref($ptrHash) =~ /HASH/) {
	  my ($tail, $keyLimit, $valLimit, $ptrLimits);

	  while (($reseller, $value) = each(%{$ptrHash})) {
		($key, $tail) = split(' ', $value, 2);
		$ptrLimits = {};

		foreach my $chunk (split(/\s*:\s*/, $tail)) {
		  if ($chunk =~ /(\S+)\s*=\s*(\S+)/) {
			$ptrLimits->{$1} = $2;
		  }
		}

		if (exists $ptrResellerInfo->{$reseller}) {
		  $ptrResellerInfo->{$reseller}->{'resource-limits'} = [$key, $ptrLimits];
		} else {
		  $ptrResellerInfo->{$reseller}={'resource-limits' => [$key, $ptrLimits]};
		}
	  }
	}
  }

  #
  # end prepare reseller's info
  #

  #
  # parse httpd.conf
  #
  if (open (INPUT,"</etc/httpd/conf/httpd.conf")) {
	binmode INPUT;
	my ($ip,$state,@vDomains,$vDomain);
	$state=0;
	while (<INPUT>) {
	  chomp;
	  next unless $_;
	  next if (/^#/);
	  if ($state==1) {
		if (/<\/VirtualHost>/) {
		  foreach $vDomain (@vDomains) {
			$vDomain=~s/^www\.//;
			$servers_ips{$vDomain}=$ip;
		  }
		  @vDomains=();
		  $state=0;
		} elsif (/^\s*Server(?:Name|Alias)\s+(.*)$/) {
		  push @vDomains,split(' ',$1);
		}
	  } else {
		if (/<VirtualHost\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})>/) {
		  $ip=$1;
		  @vDomains=();
		  $state=1;
		}
	  }
	}
	close(INPUT);
  } else {
	&printToError("Error: unamle to open '/etc/httpd/conf/httpd.conf'");
  }
  #
  # end parse httpd.conf
  #

  ##
  # Determine number of objects to dump
  ##
  my $clientCount = scalar(keys %resellers)+1;
  my $domainCount = 0;
  my %domainCounts;

  foreach $reseller (keys %resellers) {
	  $domainCounts{$reseller}=0;
  }

  unless(keys %trueUserDomains){
	  my $fileUser = makeFileParser("/etc/trueuserdomains");
	  $ptrHash = $fileUser->{'PARSE'}->('KEYSEPARATOR'=>':');
	  if (ref($ptrHash)=~/HASH/) {
		  %trueUserDomains=(%{$ptrHash});
	  }
  }
  my ($domain);
  while ( ($domain,$owner) = each %trueUserDomains) {
	  if (exists($domainCounts{$owner})) {
		  $domainCounts{$owner}++;
		  $domainCount++;
	  }
  }

  if (ref($ptrDomains) =~ /ARRAY/) {
	  $domainCount = $#{$ptrDomains};
  }

  $objDumpStatus->{'COUNTS'}->($clientCount,$domainCount);

  foreach $reseller (sort keys %resellers) {

	&printToLog("Client '$reseller' is started");

	$objDumpStatus->{'ACCOUNT'}->($reseller);
	$objDumpStatus->{'PRINT'}->();

	$item = &makeResellerNode($reseller,\@{$resellers{$reseller}});

	if (ref($item)=~/XmlNode/) {
	  $root->{'ADDCHILD'}->($item);
	  &printToLog("Client '$reseller' is successfully dumped");
	} else {
	  &printToLog("Client '$reseller' is not dumped");
	}

	$domainCount-=$domainCounts{$reseller};
	foreach $owner (@{$resellers{$reseller}}) {
	  $domainCount-=$domainCounts{$owner};
	}

	$objDumpStatus->{'COUNTS'}->(undef,$domainCount);

  }

  &printToLog("Default client is started");

  $objDumpStatus->{'ACCOUNT'}->('[default]');
  $objDumpStatus->{'PRINT'}->();

  if (@default_reseller_account) {

	$item = &makeResellerNode('',\@default_reseller_account);

	if (ref($item)=~/XmlNode/) {
	  $root->{'ADDCHILD'}->($item);
	  &printToLog("Default client is successfully dumped");
	} else {
	  &printToLog("Default client is not dumped");
	}
  }


  $objDumpStatus->{'ACCOUNT'}->('');
  $objDumpStatus->{'DOMAIN'}->('');
  $objDumpStatus->{'COUNTS'}->(0,0);
  $objDumpStatus->{'PRINT'}->();

  return $root;
}

#
# makeResellerNode
#
#   arguments:
#       $reseller - name of rteseller
#       $ptrAccounts - pointer to array of accounts
#
sub makeResellerNode {
  my ($reseller,$ptrAccounts) = @_;

  my ($account,$item,$ptrUserHash,$fileParser);

  my $root = XmlNode->new('reseller');
  $root->{'ATTRIBUTE'}->('name',$reseller);


  if ($reseller) {

	$fileParser = &makeFileParser("$installDir/users/$reseller");
	$ptrUserHash = $fileParser->{'PARSE'}->('KEYSEPARATOR'=>'=');

	my($ptrHash,$value,$key,$xmlKey,$userKey);
	$ptrHash = $ptrResellerInfo->{$reseller};

	if (ref($ptrHash)=~/HASH/) {
	  #
	  # limits
	  #
	  $value = $ptrHash->{'limits-bynumber'};
	  if ($value) {
		$root->{'ADDCHILD'}->(&makeLimitNode('max_accounts',$value));
	  }
	  if (exists $ptrHash->{$ptrHash->{'resource-limits'}}) {
		($key,$ptrHash)=@{$ptrHash->{'resource-limits'}};
	  }

	  if ($key) {
		$value = $ptrHash->{'bw'};
		if ($value=~/(\d+)/) {
		  $value = $1 * 1024*1024;
		  $root->{'ADDCHILD'}->(&makeLimitNode('max_traffic',$value));
		}
		$value = $ptrHash->{'disk'};
		if ($value=~/(\d+)/) {
		  $value = $1 * 1024*1024;
		  $root->{'ADDCHILD'}->(&makeLimitNode('disk_space',$value));
		}
	  }
	}

	while (($xmlKey,$userKey)=each(%mapLimits)) {
	  $value = $ptrUserHash->{$userKey};
	  if ($value=~/\d+/) {
		$root->{'ADDCHILD'}->(&makeLimitNode($xmlKey,$value));
	  } else {
		$root->{'ADDCHILD'}->(&makeLimitNode($xmlKey));
	  }
	}
	#
	# end limits
	#

	#
	# permissions
	#
	$ptrHash = $ptrRowResellers->{$reseller};
	if (ref($ptrHash)=~/HASH/) {
	  if (exists $ptrHash->{'create-acct'}) {
		$root->{'ADDCHILD'}->(&makePermissionNode('create_accounts'));
	  }
	  if (exists $ptrHash->{'create-dns'}) {
		$root->{'ADDCHILD'}->(&makePermissionNode('create_domains'));
	  }
	  if (exists $ptrHash->{'edit-dns'}) {
		$root->{'ADDCHILD'}->(&makePermissionNode('manage_dns'));
	  }
	}
	#
	# end permissions
	#

	$item = &makeAccountNode($reseller, $ptrUserHash, 'is reseller');
	$root->{'ADDCHILD'}->($item);

  }

  foreach $account (sort @{$ptrAccounts}) {
	$item = &makeAccountNode($account);
	$root->{'ADDCHILD'}->($item);
  }

  return $root;
}
sub getDomainName {
  my $account = shift;

  unless(keys %trueUserDomains){
	#
	# calculate $ptrTrueUserDomainsHash
	#
	my $fileParser = &makeFileParser('/etc/trueuserdomains');
	my $ptrHash = $fileParser->{'PARSE'}->('KEYSEPARATOR'=>':');
	if (ref($ptrHash)=~/HASH/) {
	  %trueUserDomains=(%{$ptrHash});
	}
  }

  my ($domain,$owner);
  while (($domain,$owner) = each %trueUserDomains ) {
	if ($owner eq $account) {
	  return $domain;
	}
  }
  return undef;
}

sub getDomainOwner {
  my $domain = shift;
  unless(keys %trueUserDomains){
	#
	# calculate $ptrTrueUserDomainsHash
	#
	my $fileParser = &makeFileParser('/etc/trueuserdomains');
	my $ptrHash = $fileParser->{'PARSE'}->('KEYSEPARATOR'=>':');
	if (ref($ptrHash)=~/HASH/) {
	  %trueUserDomains=(%{$ptrHash});
	}
  }

  return $trueUserDomains{$domain};

}

sub makeAccountNode {

  my ($account, $ptrUserHash, $isReseller) = @_;

  my ($ptrLines, $ptrHash, $ptrQuota,
	  $xmlKey, $userKey, $value, $key, $domain, $item);
  my $home = getHomeDir($account);

  my $root = XmlNode->new('account');
  $root->{'ATTRIBUTE'}->('name', $account);

  if (-T "$home/.contactemail") {

	my $fileParser = makeFileParser("$home/.contactemail");
	$ptrLines = $fileParser->{'PARSE'}->();
	if (ref($ptrLines) =~ /ARRAY/) {
	  $root->{'ATTRIBUTE'}->('email', $ptrLines->[0]||'');
	}

  } else {
	$root->{'ATTRIBUTE'}->('email', '');
  }

  unless(ref($ptrUserHash) =~ /HASH/) {
	my $fileParser = makeFileParser("$installDir/users/$account");
	$ptrUserHash = $fileParser->{'PARSE'}->('KEYSEPARATOR' => '=');
  }

  if (ref($ptrUserHash) =~ /HASH/) {
	$root->{'ATTRIBUTE'}->('date', $ptrUserHash->{'STARTDATE'}||'');
  }

  unless ($isReseller) {
	my $diskSpace = $ptrQuota->{$account};
	if ($diskSpace =~ /(\d+)/) {
	  $diskSpace = $1*1024*1024;
	  $root->{'ADDCHILD'}->(&makeLimitNode('disk_space', $diskSpace));
	}

	if (ref($ptrUserHash) =~ /HASH/) {

	  # handle limits
	  $mapLimits{'max_traffic'} = 'BWLIMIT';
	  while (($xmlKey,$userKey) = each(%mapLimits)) {
		$value = $ptrUserHash->{$userKey};
		if ($value =~ /\d+/) {
		  $root->{'ADDCHILD'}->(&makeLimitNode($xmlKey, $value));
		} else {
		  $root->{'ADDCHILD'}->(&makeLimitNode($xmlKey));
		}
	  }

	  delete $mapLimits{'max_traffic'}; ## clean map
	}
  }

  #
  # Handle domain
  #
  $domain = &getDomainName($account);

  if ($domain) {

	&printToLog("Domain '$domain' is started");

	$objDumpStatus->{'DOMAIN'}->($domain);
	$objDumpStatus->{'PRINT'}->();

	$item = &makeIpNode($domain, $ptrUserHash);
	$root->{'ADDCHILD'}->($item);

	$item = &makeDomainNode($domain, $account);
	$root->{'ADDCHILD'}->($item);

	&printToLog("Domain '$domain' is dumped");
  }

  return $root;
}

sub makeIpNode {
  my ($domain,$ptrUserHash) = @_;

  my ($ptrRows,$ip,$item,$ptrRow,$dmn);

  unless ( $ip = $servers_ips{$domain}) {
	my $fileParser = makeFileParser("$installDir/accts.db");
	$ptrRows = $fileParser->{'PARSE'}->('VALUESEPARATOR'=>',');
	foreach $ptrRow (@{$ptrRows}) {
	  $dmn = $ptrRow->[0];
	  $dmn=~s/^\s+//;
	  $dmn=~s/\s+$//;
	  if ($ptrRow->[2]=~/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
		$ip=$1;
		unless (exists($servers_ips{$domain})) {
		  $servers_ips{$domain}=$ip;
		}
	  }
	}
	unless ($ip = $servers_ips{$domain}) {
	  unless ($ip = $ptrUserHash->{'IP'}) {
		my $zone = "/var/named/$domain.db";
		my $pattern = qr/^\Q$domain\E\.\s+(?:\d+\w?\s+)IN\s+A\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/;
		if (-T $zone) {
		  if (open(INPUT,"< $zone")) {
			binmode INPUT;
			while (<INPUT>) {
			  if (/$pattern/) {
				$ip=$1;
				last;
			  }
			}
			close(INPUT);
			if ($ip) {
			  $servers_ips{$domain}=$ip;
			}
		  }
		}
	  }
	}
  }
  $item = XmlNode->new('ip', 'content' => $ip);

  return $item;
}


sub makeDomainNode {
  my ($domain,$account) = @_;

  my $root = XmlNode->new('domain');
  $root->{'ATTRIBUTE'}->('name',$domain);

  my(@subdomains,@ftpusersdirs,$subdomain,$acc,
	 $ptrRows,$ptrRow,$path,@dirs,$dir,@subdompatterns,@exclude,
	 $dumpFile,$item,$ptrProftpdRows);

  my $home = getHomeDir($account);

  #
  # subdomain dirs
  #
  unless(ref($ptrUserdomainsHash)=~/HASH/){
	my $fileParser = makeFileParser('/etc/userdomains');
	$ptrUserdomainsHash = $fileParser->{'PARSE'}->('KEYSEPARATOR'=>':');
  }
  if (ref($ptrUserdomainsHash)=~/HASH/) {
	my $pattern = qr/^(.+)\.\Q$domain\E$/;
	while ( ($subdomain,$acc) = each %{$ptrUserdomainsHash} ) {
	  next unless ($acc eq $account);
	  if ($subdomain=~/$pattern/) {
		$dir = $1;
		push @subdomains,$dir;
		push @subdompatterns,qr/^\Q$dir\E(?:\/.*)?/;
	  }
	}
  }
  #
  # end subdomain dirs
  #

  #
  # ftp dirs
  #
  if (-T "/etc/proftpd/$account") {
	my $fileParser = makeFileParser("/etc/proftpd/$account");
	$ptrProftpdRows = $fileParser->{'PARSE'}->('VALUESEPARATOR'=>':',
											   'KEYSEPARATOR'=>'',);
	if (ref($ptrProftpdRows)=~/ARRAY/) {
	  my ($subdompattern,$pattern,$is_subdomain_directory);
	  $pattern = qr/\Q$home\E\/public_html\/(.*)/;
	  foreach $ptrRow (@{$ptrProftpdRows}) {
		next unless ($ptrRow->[4] eq $account);
		$path=$ptrRow->[5];
		if ($path=~/$pattern/) {
		  $dir =$1;
		  $is_subdomain_directory=0;
		  foreach $subdompattern (@subdompatterns) {
			if ($dir=~/$subdompattern/) {
			  $is_subdomain_directory=1;
			  last;
			}
		  }
		  next if ($is_subdomain_directory);
		  push (@ftpusersdirs, $dir);
		}
	  }
	}
  }
  @subdompatterns=();
  #
  # end ftp dirs
  #

  #
  # content of domain
  #
  if (-d "$home/cgi-bin") {
	@exclude = ('cgi-bin');
  } else {
	@exclude = ();
	$dumpFile = &makeDumpFile("$dumpDir/cgi_domain_$account",
							  "$home/public_html/cgi-bin");
	$root->{'ATTRIBUTE'}->('cid_cgi',$dumpFile) if $dumpFile;
  }
  push @exclude,@subdomains,@ftpusersdirs;
  $dumpFile = &makeDumpFile("$dumpDir/docroot_domain_$account",
							"$home/public_html",['.'],\@exclude);
  $root->{'ATTRIBUTE'}->('cid_docroot',$dumpFile) if $dumpFile;

  #
  # end content of domain
  #

  if ($item = &makeAnonFtpdNode($account)) {
	$root->{'ADDCHILD'}->($item);
  }

  &addFtpUserNodes($root,$domain,$account,
				   \@subdomains,\@ftpusersdirs,$ptrProftpdRows);

  if ($noContent) {
	return $root;
  }

  &addProtectedDirs($root,$domain,$account,
					\@subdomains);

  &addSubDomains($root, $domain, $account,
				 \@subdomains);

  &addAddOnDomains($root,$domain,$account);

  addMailNode($root, $domain, $account);

  &addDatabases($root,$account);

  return $root;
}

sub addDatabases {
  my ($root,$account) = @_;

  my ($ptrRow,$database,$item,$dumpFile,$sql,@dbNames);

  unless((ref($wrapUserMysql)=~/HASH/) &&
		 (ref($wrapUserMysql->{'EXECUTE'})=~/CODE/)){

	$wrapUserMysql = getDbConnect('mysql', 'root', undef, 'mysql', 'localhost');
  }

  my $datadir;
  $sql="SHOW variables LIKE 'datadir'";
  if ($wrapUserMysql->{'EXECUTE'}->($sql) and $ptrRow = $wrapUserMysql->{'FETCHROW'}->()){
    $datadir = $ptrRow->[1];
  }
  $wrapUserMysql->{'FINISH'}->();

  $sql="SELECT DISTINCT db FROM db WHERE db like '".$account."\_%' or db like '".$account."\\\_%'";
  if ($wrapUserMysql->{'EXECUTE'}->($sql)) {
	while ($ptrRow = $wrapUserMysql->{'FETCHROW'}->()) {
	  push @dbNames,$ptrRow->[0];
	}
  }
  $wrapUserMysql->{'FINISH'}->();

  foreach $database (@dbNames) {
	$database =~ s/\\_/_/g;
	next unless -d $datadir.$database;
	$item = XmlNode->new('database');
	$item->{'ATTRIBUTE'}->('name',$database);
	$item->{'ATTRIBUTE'}->('type','mysql');
	$item->{'ATTRIBUTE'}->('version', Db::MysqlUtils::getVersion());

	printToLog("Started $database dumping...");

	if ($dumpFile = &makeDumpDb($database, 'mysql', 'root')) {
	  $item->{'ATTRIBUTE'}->('cid',$dumpFile);
	}

	addDbUsers($item, $database, $wrapUserMysql);

	$root->{'ADDCHILD'}->($item);

	printToLog("done");
  }
}

sub addSubDomains {

  my($root, $domain, $account, $ptrSubDomains) = @_;

  my($nodeSubDomain, $subdomain, $dumpFile, $subdomainRoot, $fileParser,
	 $ptrRows, $ptrRow, $item, @exclude);

  my $home = getHomeDir($account);


  foreach $subdomain (@{$ptrSubDomains}) {

	&printToLog("Subdomain '$subdomain.$domain' ...", 1);

	$subdomainRoot = "$home/public_html/$subdomain";

	$nodeSubDomain = XmlNode->new('subdomain');
	$nodeSubDomain->{'ATTRIBUTE'}->('name', $subdomain, 'noencode');
	$root->{'ADDCHILD'}->($nodeSubDomain);

	#
	# content
	#
	if (-d "$subdomainRoot/cgi-bin") {

	  if ($dumpFile = &makeDumpFile("$dumpDir/cgi_subdomain_$subdomain.$domain", $subdomainRoot, 'cgi-bin')) {
		$nodeSubDomain->{'ATTRIBUTE'}->('cid_cgi', $dumpFile);
	  }
	  @exclude=('cgi-bin');

	} else {
	  @exclude=();
	}

	if ($dumpFile = &makeDumpFile("$dumpDir/docroot_subdomain_$subdomain.$domain", $subdomainRoot, ['.'], \@exclude)) {
	  $nodeSubDomain->{'ATTRIBUTE'}->('cid_docroot', $dumpFile);
	}

	#
	# ftp user
	#
	if (-T "/etc/proftpd/$account") {
	  $fileParser = &makeFileParser("/etc/proftpd/$account");
	  $ptrRows = $fileParser->{'PARSE'}->('VALUESEPARATOR' => ':');
	  if (ref($ptrRows) =~ /ARRAY/) {
		foreach $ptrRow (@{$ptrRows}) {
		  if ( ($ptrRow->[0] eq $subdomain) && ($ptrRow->[4] eq $account) ) {
			if ($ptrRow->[1]) {
			  $item = &makeFtpUserNode($subdomain, $ptrRow->[1]);
			  $nodeSubDomain->{'ADDCHILD'}->($item);
			}
			last;
		  }
		}
	  }
	}

	# Dump mail system and addon domains: if there is 1 addon domain for
	# given subdomain - dump mail system of ADDON DOMAIN, not subdomain.
	# This case reflects common usage of addon domains in cPanel...
	{
		my $maildomain = "$subdomain.$domain";

		my @addondomains = addAddOnDomains($nodeSubDomain, "$subdomain.$domain", $account);
		if (0 == $#addondomains) {
			$maildomain = pop @addondomains;
		}

		addMailNode($nodeSubDomain, $maildomain, $account);
	}

	&printToLog(' OK');
  }
}

my %apacheAliases = ();

###
# Returns array of addon domain names that has been added.
# 
# @return array
###
sub addAddOnDomains {
  my ($root,$domain,$account)=@_;
  my ($userdomain,$owner,%userdomains,$maindomain,%aliases,$alias,$item,@found);

  unless (keys %apacheAliases){
	unless(open (INPUT,"/etc/httpd/conf/httpd.conf")){
	  return ();
	}
	binmode INPUT;
	my ($state,$vserver,%valiases);
	$state=0;
	while (<INPUT>) {
	  chomp;
	  if ($state==1) {
		if (/^\s*<\/VirtualHost\s*>/) {
		  if ($vserver && (keys %valiases)) {
			push @{$apacheAliases{$vserver}},keys %valiases;
		  }
		  $vserver='';
		  %valiases=();
		  $state=0;
		} elsif (/^\s*ServerName\s+(\S+)/) {
		  $vserver=$1;
		  $vserver=~s/^www\.//;
		} elsif (/^\s*ServerAlias\s+(.*)/) {
		  map {s/^www\.//;$valiases{$_}=1} split(' ',$1);
		}
	  } else {
		if (/^\s*<VirtualHost\s.*>/) {
		  $state=1;
		  $vserver='';
		  %valiases=();
		}
	  }
	}
	close (INPUT);
  }
  unless (exists $apacheAliases{$domain}) {
	return ();
  }
  unless(ref($ptrUserdomainsHash)=~/HASH/){
	my $fileParser = &makeFileParser('/etc/userdomains');
	$ptrUserdomainsHash = $fileParser->{'PARSE'}->('KEYSEPARATOR'=>':');
  }

  $maindomain = &getDomainName($account);

  if (ref($ptrUserdomainsHash)=~/HASH/) {
	my $pattern = qr/\Q$maindomain\E$/;
	while (($userdomain,$owner)=each(%{$ptrUserdomainsHash})) {
	  next unless $owner eq $account;
	  next if ($userdomain=~/$pattern/); ## exclude subdomains
	  $userdomains{$userdomain}=1;
	}
  }

  %aliases = map {$_=>1} @{$apacheAliases{$domain}};
  foreach $alias (keys %aliases) {
	next unless exists $userdomains{$alias};
	$item = XmlNode->new('addondomain', 'content' => $alias);
	$root->{'ADDCHILD'}->($item);
	push @found, $alias;
  }

  return @found;
}

sub addMailNode {
  my ($root, $domain, $account) = @_;
  my ($mailNode,$ptrRows,$ptrRow,$default,$item,$box,
	  $ptrHash,$value,$password,$dumpFile,$ptrAliases,$src,$dst,
	  $from);
  my $home = getHomeDir($account);

  printToLog("Mail for $domain migration started...");
  $mailNode = XmlNode->new('mail');
  $ptrAliases=[];
  if (-T "/etc/valiases/$domain") {
	my $fileParser = &makeFileParser("/etc/valiases/$domain");
	$ptrAliases=$fileParser->{'PARSE'}->('VALUESEPARATOR'=>':');
	if (ref($ptrAliases)=~/ARRAY/) {
	  foreach $ptrRow (@{$ptrAliases}) {
		if ($ptrRow->[0] eq '*') {
		  if ($ptrRow->[1]=~/\s*([^@]+\@[^@+]+)\s*/) {
			$default=$1;
			last;
		  } elsif ($ptrRow->[2]=~/blackhole/) {
			$default=':blackhole:';
			last;
		  } elsif ($ptrRow->[2]=~/fail/) {
			$default=':fail:'.$ptrRow->[3];
			last;
		  }
		}
	  }
	}
  }

  if ($default) {
	$item = XmlNode->new('default');
	$mailNode->{'ADDCHILD'}->($item);
	if ($default eq ':blackhole:') {
	  $item->{'ATTRIBUTE'}->('target','ignore');
	} elsif ($default =~ /:fail:(.*)/) {
	  $item->{'ATTRIBUTE'}->('target','fail');
	  $item->{'CONTENT'}->($1);
	} else {
	  $item->{'ATTRIBUTE'}->('target','email');
	  $item->{'CONTENT'}->($default);
	}
  }

  printToLog("Examining $home/etc/$domain");

  if (-d "$home/etc/$domain" && -T "$home/etc/$domain/passwd") {
	my $fileParser = makeFileParser("$home/etc/$domain/passwd");
	$ptrRows=$fileParser->{'PARSE'}->('VALUESEPARATOR'=>':');

	if (ref($ptrRows)=~/ARRAY/) {
	  foreach $ptrRow (@{$ptrRows}) {
		$box = $ptrRow->[0];
		$item = XmlNode->new('mailname');
		$item->{'ATTRIBUTE'}->('name',$box);
		$mailNode->{'ADDCHILD'}->($item);

		printToLog("migrating " .$box . "@" ."$domain");

		#
		# password
		#
		$password='';
		my $shadowParser = makeFileParser("$home/etc/$domain/shadow");
		$ptrHash = $shadowParser->{'PARSE'}->('KEYSEPARATOR'=>':',
											  'VALUESEPARATOR'=>'');

		if (ref($ptrHash)=~/HASH/) {
		  if ($value=$ptrHash->{$box}) {
			($password)=split(':',$value);
		  }
		}
		$item->{'ATTRIBUTE'}->('password',$password);
		#
		# end password
		#

		#
		# quota
		#
		if ( -T "$home/etc/$domain/quota") {
		  my $quotaParser = makeFileParser("$home/etc/$domain/quota");
		  $ptrHash = $quotaParser->{'PARSE'}->('KEYSEPARATOR' => ':', 'VALUESEPARATOR' => '');
		  if (ref($ptrHash) =~ /HASH/ && ($value = $ptrHash->{$box})) {
			$item->{'ATTRIBUTE'}->('quota', sprintf("%.0f", $value));
		  }
		}
		#
		# end quota
		#
		if ($dumpFile = makeDumpFile("$dumpDir/mailname_$box.$domain",
									  "$home/mail/$domain/$box")) {
		  $item->{'ATTRIBUTE'}->('cid',$dumpFile);
		}
	  }
	}
  }
  if (@{$ptrAliases}) {
	addForwarders($mailNode, $ptrAliases);
	addAutoresponders($mailNode, $account, $ptrAliases);
  }

  $root->{'ADDCHILD'}->($mailNode);
  printToLog("Mail for $domain migration finished...");
}

sub addAutoresponders {
  my ($mailNode, $account, $ptrAliases) = @_;
  my $home = getHomeDir($account);

  foreach my $ptrRow (@{$ptrAliases}) {

	my $src = $ptrRow->[0];
	my $dst = $ptrRow->[1];
	chomp $src;

	next unless ($dst =~ /^\s*\|.*\/cpanel\/bin\/autorespond/);
	next unless ($src =~ /^([^@]+@[^@]+)$/);
	next unless (open(INPUT, "$home/.autorespond/$src"));
	binmode INPUT;

	my $autoresponderNode = XmlNode->new('autoresponder');
	$mailNode->{'ADDCHILD'}->($autoresponderNode);
	$src =~ /^([^@]+)@/;
	$autoresponderNode->{'ATTRIBUTE'}->('mailname', $1);

	while (<INPUT>) {
	  chomp;
	  last unless $_;

	  if (/^From:.+<(.+)>/) {
		$autoresponderNode->{'ATTRIBUTE'}->('from', $1) if ($src ne $1);
	  } elsif (/^Content-type:\s+text\/([^;]+);\s+charset=(.+)/) {
		$autoresponderNode->{'ATTRIBUTE'}->('type', $1);
		$autoresponderNode->{'ATTRIBUTE'}->('charset', $2);
	  } elsif (/^Subject:\s*(.*)/) {
		$autoresponderNode->{'ATTRIBUTE'}->('subject', $wrapBase64->{'ENCODE'}->($1));
	  }
	}

	$autoresponderNode->{'CONTENT'}->($wrapBase64->{'ENCODE'}->(join('',(<INPUT>))));

	close(INPUT);
  }
}

sub addForwarders {
  my ($mailNode, $ptrAliases) = @_;

  foreach my $ptrRow (@{$ptrAliases}) {

	my $src = $ptrRow->[0];
	my $dst = $ptrRow->[1];

	next unless ($src =~ /^([^@]+)@[^@]+$/);
	my $src = $1;

	$dst =~ s/^\s+//;
	$dst =~ s/\s+$//;
	next if ($dst =~ /^['"]?\|/); # autoresponder

	foreach my $email (split /,/, $dst) {
		next unless ($email =~ /[^@]+@[^@]+/);
		my $forwardNode = XmlNode->new('forward');
		$mailNode->{'ADDCHILD'}->($forwardNode);
		$forwardNode->{'ATTRIBUTE'}->('mailname', $src);
		$forwardNode->{'ATTRIBUTE'}->('redirect', $email);
	}
  }
}

sub addProtectedDirs {

  my ($root, $domain, $account, $ptrSubDomains) = @_;

  my ($cmd, $exclude, $pdir, $find, $item, $title, $ptrHash,
	  $nodePdir, $name, $password);

  my $home = getHomeDir($account);
  my $rootDir = "$home/.htpasswds";
  unless(-d $rootDir){
	return;
  }

  $find = AgentConfig::findBin();
  $cmd = " cd $rootDir; $find . ";
  if (@{$ptrSubDomains}) {
	$exclude = join(' -o ',map {"-path './$_' -prune"} @{$ptrSubDomains});
	$cmd .= "\\( $exclude \\) -o";
  }
  $cmd .= " \\( -type f -name passwd -printf '\%h\\n' \\)";
  foreach $pdir (`$cmd`) {
	chomp $pdir;
	$pdir=~s/^\.\///;
	$nodePdir = XmlNode->new('pdir');
	$nodePdir->{'ATTRIBUTE'}->('name',$pdir);
	$root->{'ADDCHILD'}->($nodePdir);

	$title = '';
	if (-T "$home/public_html/$pdir/.htaccess") {
	  my $fileParser = makeFileParser("$home/public_html/$pdir/.htaccess");
	  $ptrHash = $fileParser->{'PARSE'}->();

	  if (ref($ptrHash)=~/HASH/) {
		if (exists($ptrHash->{'AuthName'})) {
		  if ($ptrHash->{'AuthName'}=~/"([^"]+)"/) {
			$title = $1;
		  }
		}
	  }
	}

	$nodePdir->{'ATTRIBUTE'}->('title',$title);
	my $fileParser = makeFileParser("$rootDir/$pdir/passwd");
	$ptrHash = $fileParser->{'PARSE'}->('KEYSEPARATOR'=>':');
	if (ref($ptrHash)=~/HASH/) {
	  while (($name,$password)=each(%{$ptrHash})) {
		$item = XmlNode->new('pduser');
		$nodePdir->{'ADDCHILD'}->($item);
		$name =~ s/ /_/;
		$item->{'ATTRIBUTE'}->('name',$name);
		if ($password !~ /^[\x20-\x7f]*$/) {
		  $item->{'ATTRIBUTE'}->('password', $wrapBase64->{'ENCODE'}->($password));
		  $item->{'ATTRIBUTE'}->('encoding', 'base64');
		} else {
		  $item->{'ATTRIBUTE'}->('password',$password);
		}
	  }
	}
  }
}

sub addFtpUserNodes
  {
	my ($root, $domain, $account, $ptrSubDomains, $ptrFtpDirs, $ptrProftpdRows) = @_;

	my ($password, @users);

	my $home    = getHomeDir($account);
	my $pattern = qr/\Q$home\E\/public_html\/(.*)/;

	foreach my $ptrRow (@{$ptrProftpdRows}) {

	  if ($ptrRow->[4] eq $account) {
		if ($ptrRow->[5] =~ /$pattern/) {
		  my $subdir = $1;
		  my $pass = ($ptrRow->[1] ne '*') ? $ptrRow->[1] : '';

		  if (grep(($_ eq $subdir), @{$ptrFtpDirs})) {
			push @users, [$ptrRow->[0], $pass, $subdir];
		  }
		}
	  }

	  if (!$password && ($ptrRow->[0] eq $account)) {
		$password = $ptrRow->[1];
	  }
	}

	if ('*' eq $password) {
	  $password = getSystemPassword($account);
	  if ('*' eq $password) {
		$password = '';
	  }
	}

	# makes main domains's ftp user node
	$root->{'ADDCHILD'}->(makeFtpUserNode($account, $password));

	# other ftp users of domain
	foreach my $ptrRow (@users) {

	  my ($acc, $pass, $path) = @{$ptrRow};
	  my $ftpUserNode = makeFtpUserNode($acc, $pass);

	  if ($path) {
		$ftpUserNode->{'ATTRIBUTE'}->('directory', $path);
		if (my $dumpFile = makeDumpFile("$dumpDir/ftpuser_$acc.$account", "$home/public_hmtl/$path")) {
		  $ftpUserNode->{'ATTRIBUTE'}->('cid', $dumpFile);
		}
	  }

	  $root->{'ADDCHILD'}->($ftpUserNode);
	}
  }

sub makeFtpUserNode
  {
	my ($account, $password) = @_;
	my $item = XmlNode->new('ftpuser');
	$item->{'ATTRIBUTE'}->('name', $account);
	# User may be locked
	if ($password && $password !~ /^\!/) {
	  $item->{'ATTRIBUTE'}->('password', $password);
	}
	return $item;
  }

sub makeAnonFtpdNode {
  my $account = shift;
  my ($node,$publicFtp,$pureFtpd);
  my $home = getHomeDir($account);

  $publicFtp="$home/public_ftp";
  unless(-d $publicFtp){
	return undef;
  }
  $node = XmlNode->new('anonftp');

  if ($dumpFile = &makeDumpFile("$dumpDir/anonftp_pub_$account",
								$publicFtp,['.'],['incoming'])) {
	$node->{'ATTRIBUTE'}->('cid',$dumpFile);
  }
  if ($dumpFile = &makeDumpFile("$dumpDir/anonftp_incoming_$account",
								"$publicFtp/incoming")) {
	$node->{'ATTRIBUTE'}->('cid_incoming',$dumpFile);
  }
  #
  # prepare anonftps
  #
  $pureFtpd = '/etc/pure-ftpd';
  if (-d $pureFtpd) {
	unless(keys %anonftps){
	  my ($link,$linkTo,$fullPath);
	  if (opendir(FTPD,$pureFtpd)) {
		foreach $link (readdir(FTPD)) {
		  $fullPath = "$pureFtpd/$link";
		  next unless (-l $fullPath);
		  $linkTo = readlink($fullPath);
		  if ($linkTo=~/\Q$home\E\/([^\/]+)\/public_ftp/) {
			push @{$anonftps{$1}},$link;
		  }
		}
		closedir(FTPD);
	  }
	}
  }
  #
  # end prepare anonftps
  #

  #
  # check permissions
  #
  if (exists($anonftps{$account}) &&
	  ( (stat($publicFtp))[2] & 05)
	 ) {

	$node->{'ATTRIBUTE'}->('pub','true');

	if ( (stat("$publicFtp/incoming"))[2] & 03 ) {

	  $node->{'ATTRIBUTE'}->('incoming','true');

	}
  }

  #
  # end check permissions
  #

  return $node;
}

sub makePermissionNode {
  my($name)=@_;
  my $item = XmlNode->new('permission');
  $item->{'ATTRIBUTE'}->('name',$name);
  return $item;
}

sub makeLimitNode {
  my($name,$value)=@_;
  unless(defined($value) && ($value !~ '/^\s*$/')){
	$value = '-1';
  }
  my $item = XmlNode->new('limit', 'content' => $value);
  $item->{'ATTRIBUTE'}->('name',$name);
  return $item;
}

sub getCpanelVersion {
  my ($engineRoot, $util);
  if ($engineRoot = $cPanelConfig{'engineroot'}) {
	unless (-d $engineRoot) {
	  return;
	}
	$util="$engineRoot/cpanel";
	unless (-x $util) {
	  return;
	}
	my @outs = `$util`;
	foreach my $line (@outs) {
	  chomp $line;
	  if ($line=~/cPanel\s+\[(\d+)\.(\d+).\d+/) {
		if ($1 ne '9' && $1 ne '10') {
		  return;
		}
		return "$1.$2";
	  }
	}
  } elsif (-f "$installDir/cpanel.config") {
	return '9';
  }
}

sub printAgentStatus {
  my $root = XmlNode->new('agent-status');

  unless (getCpanelVersion() && defined AgentConfig::iconvBin()) {
    my $item;
	if (defined AgentConfig::iconvBin()) {
	  $item = XmlNode->new('wrong-platform', 'content' => '');
	} else {
	  $item = XmlNode->new('wrong-platform', 'content' => 'no iconv found on the source host');
	}

	$root->{'ADDCHILD'}->($item);
  }

  $root->serialize(\*STDOUT);
}


sub loadMainConfig
  {
	my $pathToConf = shift || '';
	my $ptrHash    = shift;

	unless (-T $pathToConf) {
	  foreach my $prefix ('./', '/var/cpanel/') {
		$pathToConf = $prefix . 'cpanel.config';
		if (-T $pathToConf) {
		  last;
		}
	  }
	}

	if (-T $pathToConf) {
	  if (ref($ptrHash) =~ /HASH/) {
		if ($pathToConf =~ /(.+)\/[^\/]+/) {
		  $ptrHash->{'installDir'} = $1;
		}
		if (open(CF, "<$pathToConf")) {
		  binmode CF;
		  my ($key, $value);
		  while (my $line = <CF>) {
			chomp $line;
			next unless $line;
			next if ($line =~ /^#/);
			($key, $value) = split(/\s*=\s*/, $line, 2);
			$key =~ s/^\s+//;
			$ptrHash->{$key} = $value;
		  }
		  close (CF);
		}
	  }
	} else {
	  $pathToConf = '';
	}
	return $pathToConf;
  }

sub printHelp {

  print <<"HELP";

Usage:
$0 <options>

Options:
-c |--config=<path>        path to cpanel.conf

-s |--get-status           get status of the agent
-dc|--dump-accounts=<list> a coma separated list of resellers to dump
-dd|--dump-domains=<list>  a coma separated list of customers to dump
-da|--dump-all             make a full dump

-lc|--get-content-list     get list of content files
-nc|--no-content           do not make content files
-nz|--no-compress          do not compress content files

-h |--help                 this help

HELP
}


