package Mail::Qmail;

use strict;
use warnings;

sub new {
  my $self = { };
  bless($self);

  return $self;
}

sub get_rcpthosts {
  my $self = shift;
  open(RCPTHOSTS, '<', '/var/qmail/control/rcpthosts') or  return(undef, 0, "Unable to open rcpthosts file.");
  my @results;
  while (<RCPTHOSTS>) {
    chomp;
    push(@results, $_);
  }
  close(RCPTHOSTS);
  return @results;
}

sub get_virtualdomains {
  my $self = shift;
  open(VDOMS, '<', '/var/qmail/control/virtualdomains') or return(undef, 0, "Unable to open virtualdomains file.");

  my @results;
  while (<VDOMS>) {
    chomp;
    my ($domain, $user) = split(':');
    push(@results, { domain => $domain, user => $user } );
  }
  close(VDOMS);
  return @results;
}

sub get_info_from_assign {
  my ($self, $assign) = @_;
  my (@lines, @results);
  $assign ||= "/var/qmail/users/assign";
  open(ASSIGN, "< $assign");
  while(<ASSIGN>){
    chomp;
    push @lines, $_;
  }
  close(ASSIGN);
  foreach my $line (@lines){
    my @fields = split(":", $line);
    next unless defined $fields[4];
    my @parts = split("/", $fields[4]);
    if(defined($parts[5])){
      push(@results, { domain => $parts[4], mailname => $parts[5], directory => $fields[4]});
    }
  }
  return @results;
}

sub parse_dot_qmail_file {
  my ($self, $directory) = @_;
  my (@lines, %results);
  my $res = \%results;
  my $dotQmail = $directory."/.qmail";
  return unless(-e $dotQmail);
  open(DOTQMAIL, "< $dotQmail");
  while(<DOTQMAIL>) {
    chomp;
    push @lines, $_;
  }
  close(DOTQMAIL);

  foreach my $line(@lines) {
    if ($line=~/\&(.*)/) {
      push @{$res->{redirect}}, $1;
    }
  }

  return $res;
}

sub get_passwd_by_dir {
  my ($self, $directory, $passwd_file) = @_;
  $passwd_file ||= "/var/qmail/users/poppasswd";

  my $result;

  open(PASSWD, "< $passwd_file");
  while(<PASSWD>) {
    chomp;
    my @fields = split(":");
    if($fields[3] eq $directory) {
      $result = $fields[1];
    }
  }
  close(PASSWD);

  return $result;
}

1;
