package Mail::Sendmail;

use strict;
use warnings;

sub new {
  my $self = { };
  bless($self);

  return $self;
}

sub getLocalDomains {
  my ($self, $config) = @_;
  $config ||= '/etc/mail/local-host-names';
  open (LOCAL, "< $config") or return (undef, 0, "Unable to open local-host-names file");

  my @results;
  while (<LOCAL>) {
    chomp;
    push (@results, $_);
  }

  close (LOCAL);
  return @results;
}

sub getDomainMails {
  my ($self, $domain, $virt_user_table) = @_;
  $virt_user_table ||='/etc/mail/virtusertable';

  my %mailnames;

  open (VIRT, " < $virt_user_table");

  while (<VIRT>) {
    if ($_=~/(.*)\@$domain\s+(.*)/){
      $mailnames{$1} = $2;
    }
  }
  close (VIRT);

  return \%mailnames;

}

sub collectMailInfo {
  my ($self, $mailnames) = @_;
  my $mail_info;
  foreach my $mail_unit (keys %{$mailnames}) {
    my $dest_mail = $mailnames->{$mail_unit};
    my $mailPtr = $self->parsePasswdShadow($dest_mail);
    if (defined $mailPtr->{$dest_mail}){
      $mail_info->{$mail_unit}->{'directory'} = $mailPtr->{$dest_mail}->{'directory'};
      $mail_info->{$mail_unit}->{'password'} = $mailPtr->{$dest_mail}->{'password'};
    }

    if ($dest_mail=~/[.*]\@[.*]/){
      $mail_info->{$mail_unit} = {'redirect' => $dest_mail};
    }

    my $aliasPtr = $self->parseAliases($dest_mail);
    if (defined $aliasPtr->{$dest_mail}) {
      foreach my $alias (@{$aliasPtr->{$dest_mail}}) {
        if (defined $self->parsePasswdShadow($alias)) {
          push @{$mail_info->{$mail_unit}->{'mailgroup'}}, $self->getMailByDest($alias);
        }
      }
    }
  }
  return $mail_info;
}

sub getMailByDest {
  my ($self, $mail) = @_;
  my $virt_user_table ||= '/etc/mail/virtusertable';

  my $result;
  open (VIRT, " < $virt_user_table");
  while (<VIRT>) {
    if($_=~/(.*)\@\w+\s+$mail/){
      $result = $1;
    }
  }
  close(VIRT);
  return $result;
}

sub parseAliases {
  my ($self, $mail) = @_;
  my $aliases_file = '/etc/mail/aliases';

  my %params;

  open (ALIASES, "< $aliases_file");
  while (<ALIASES>) {
    chomp;
    if ($_=~/$mail\:\s+(.*)/) {
      @{$params{$mail}} = split(/,/, $1);
    }
  }
  close(ALIASES);

  return \%params;
}

sub parsePasswdShadow {
  my ($self, $uid) = @_;

  my $shadow_file = '/etc/shadow';
  my $passwd_file = '/etc/passwd';

  my (%shadow,%passwd, %user_params, @params, $user, $password, @shadowParams);
  open (SHADOW, "< $shadow_file");
  while (<SHADOW>) {
    chomp;
    split(/:/, $_);
    #FIXME - remove @_->[0] at the begining of hash value
    push @{$shadow{@_->[0]}}, @_;
  }
  close (SHADOW);

  open (PASSWD, "< $passwd_file");
  while (<PASSWD>) {
    chomp;
    split(/:/, $_);
    #FIXME - go to previous fix me
    push @{$passwd{@_->[0]}}, @_;
  }
  close (PASSWD);

  while (($user, @params) = each %passwd) {
    if ($user eq $uid) {
      if (%shadow and defined($shadow{$user})) {
        @shadowParams = @{$shadow{$user}};
        $password = $shadowParams[1];
      }
      $user_params{$user} = {'shell' => $params[0]->[6], 'password' => $password, 'directory' => $params[0]->[5]};
    }
  }

  return \%user_params;
}

1;
