package MigrationDumpStatus;

use strict;
use DumpStatus;
use Logging;
use XmlNode;

use vars qw|@ISA|;

@ISA = qw|DumpStatus|;

sub _init {
  my ($self, $filename) = @_;
  $self->{filename} = $filename;

  Logging::info("-" x 60);
  Logging::info("Migration status reporting initialized.");
  Logging::info("Status file: $filename");
  Logging::info("-" x 60);
}

sub start {
  my ($self, $clients, $domains) = @_;
  $self->{current_client} = undef;
  $self->{current_domain} = undef;
  $self->{clients_count} = $clients;
  $self->{domains_count} = $domains;

  $self->_publish();
}

sub startDomain {
  my ($self, $domain) = @_;
  $self->{domains_count}--;
  $self->{current_domain} = $domain;

  $self->_publish();
}

sub startClient {
  my ($self, $client) = @_;
  $self->{clients_count}--;
  $self->{current_client} = $client;

  $self->_publish();
}

sub finishObjects {
  my ($self) = @_;

  $self->{current_client} = undef;
  $self->{current_domain} = undef;
  $self->_publish();
}

sub finish {
}

sub _publish {
  my ($self) = @_;
  my $tmpfile = $self->{filename} . ".tmp";

  open STATUS, ">$tmpfile";
  binmode STATUS;

  my $xmlRoot = XmlNode->new('agent-dump-status',
                             'children' => [XmlNode->new('account',
                                                         'content' => $self->{current_client}),
                                            XmlNode->new('domain',
                                                         'content' => $self->{current_domain}),
                                            XmlNode->new('accounts-left',
                                                         'content' => $self->{clients_count}),
                                            XmlNode->new('domains-left',
                                                         'content' => $self->{domains_count})]);

  $xmlRoot->serialize(\*STATUS);
  close(STATUS);

  rename($tmpfile, $self->{filename});
}

1;

# Local Variables:
# mode: cperl
# cperl-indent-level: 2
# indent-tabs-mode: nil
# tab-width: 4
# End:
